
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_CLIENT_URI_SCRIPT_BODY_PARSER_H
#define PG_CLIENT_URI_SCRIPT_BODY_PARSER_H

#include "xstd/RegEx.h"
#include "client/BodyParserFarm.h"
#include "client/BodyParser.h"

class CltXact;
class CltCfg;

// searches for anything that looks like a URI_SCRIPT and
// forwards them to the owner
class UriScriptBodyParser: public BodyParser {
	public:
		static BodyParser *GetOne(CltXact *anOwner, const CltCfg *aCfg);

	public:
		UriScriptBodyParser();

		void configure(CltXact *anOwner, const CltCfg *aCfg);

		virtual void reset();
		virtual BodyParserFarm &farm() const;

		virtual Size parse(const ParseBuffer &data);
		virtual void noteLeftovers(const ParseBuffer &data);
		virtual void noteOverflow(const ParseBuffer &data);

	protected:
		static void ConfigureScriptFunRe(RegEx &re, const String &name);

		void parseOpen(const ParseBuffer &buf);
		void parseClose(const ParseBuffer &buf);
		void parseBody(const ParseBuffer &buf);
		Size parseAny(const ParseBuffer &buf);

		void parseFrameFun();
		void parseRspMsg();

		bool parseConstValue(String &uri, const RegEx::Match &m);
		bool parseNameValue(String &uri, const RegEx::Match &nm);
		bool findNameValue(String &uri, RegEx &re, const String &name);

		bool findNextRe(const RegEx &re, RegEx::Matches &matches, const char *&start, const char *end = 0);
		void parseUris(const char *beg, const char *end);
		void forwardUri(const String &uri);

	protected:
		static RegEx TheUriRe;
		static RegEx TheRspMsgRe;
		static RegEx TheFramelessFunRe;
		static RegEx TheFramefullFunRe;

		const CltCfg *theCfg;

		const char *theScriptBeg;
		const char *theScriptEnd;
		const RegEx *theFrameFunRe;
		int theRspMsgIdx;

		enum { stNone = 0, stOpen, stClose, stBody, stDone } theState;

	private:
		void resetSelf();

	private:
		static BodyParserFarmT<UriScriptBodyParser> TheParsers;
};

#endif

