
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include <fstream.h>

#include "xml/XmlAttr.h"
#include "loganalyzers/Stex.h"
#include "loganalyzers/ReportBlob.h"
#include "loganalyzers/BlobDb.h"
#include "loganalyzers/RepOpts.h"
#include "loganalyzers/PhaseInfo.h"
#include "loganalyzers/PhaseTrace.h"
#include "loganalyzers/RptmTraceFig.h"


RptmTraceFig::RptmTraceFig(): thePhase(0), theTrace(0) {
}

void RptmTraceFig::stats(const Stex *aStex, const PhaseInfo *aPhase) {
	thePhase = aPhase;
	theTrace = &thePhase->trace();
	Assert(theTrace);
	moreStats(aStex);
}

void RptmTraceFig::moreStats(const Stex *stex) {
	if (const TmSzStat *aggr = stex->aggr(*thePhase)) {
		if (aggr->count())
			theStexes.append(stex);
	}
}

void RptmTraceFig::setCtrlOptions() {
	ReportTraceFigure::setCtrlOptions();
	*theCtrlFile << "set size 1.0,0.5" << endl;
	*theCtrlFile << "set xlabel 'minutes'" << endl;
	*theCtrlFile << "set ylabel 'msec'" << endl;
}

int RptmTraceFig::createCtrlFile() {
	if (ReportTraceFigure::createCtrlFile() < 0)
		return -1;

	// create plot command
	{for (int s = 0; s < theStexes.count(); ++s) {
		if (s == 0)
			*theCtrlFile << "plot '-' ";
		else
			*theCtrlFile << ",\\" << endl << "\t '-' ";
		*theCtrlFile << "title '" << theStexes[s]->name()
			<< "' with lines lt " << (theStexes.count() - s);
	}}
	*theCtrlFile << endl;

	// dump data to plot
	int pointCount = 0;
	{for (int s = 0; s < theStexes.count(); ++s) {
		if (s)
			*theCtrlFile << 'e' << endl; // note: two empty lines do not work
		pointCount += dumpDataLines(theStexes[s]);
	}}
	return pointCount;
}

int RptmTraceFig::dumpDataLines(const Stex *stex) {
	int pointCount = 0;
	for (int i = 0; i < theTrace->count(); ++i)
		pointCount += dumpDataLine(stex, theTrace->winPos(i), theTrace->winStats(i));
	return pointCount;
}

int RptmTraceFig::dumpDataLine(const Stex *stex, Time stamp, const StatIntvlRec &r) {
	const AggrStat &stat = stex->trace(r)->time();
	if (stat.count()) {
		dumpTime(stamp);
		*theCtrlFile << ' ' << stat.mean() << endl;
		return 1;
	}
	return 0;
}

